@@ -6,9 +6,9 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as
- * published by the Free Software Foundation, with linking exception;
+ * published by the Free Software Foundation, with linking exceptions;
  * either version 2.1 of the License, or (at your option) any later
- * version.
+ * version. See COPYING file for details.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -21,23 +21,36 @@
  * 02111-1307, USA.
  *)
 
-type t = { mutable locked : bool; mutable waiting : unit Lwt.t list  }
+open Lwt
 
-let create () = { locked = false; waiting = [] }
+type t = { mutable locked : bool; mutable waiters : unit Lwt.u Lwt_sequence.t  }
+
+let create () = { locked = false; waiters = Lwt_sequence.create () }
 
 let rec lock m =
   if m.locked then begin
-    let res = Lwt.wait () in
-    m.waiting <- res :: m.waiting;
-    Lwt.bind res (fun () ->
-    lock m)
+    let (res, w) = Lwt.task () in
+    let node = Lwt_sequence.add_r w m.waiters in
+    Lwt.on_cancel res (fun _ -> Lwt_sequence.remove node);
+    res
   end else begin
     m.locked <- true;
     Lwt.return ()
   end
 
 let unlock m =
-  let w = m.waiting in
-  m.waiting <- [];
-  m.locked <- false;
-  List.iter (fun t -> Lwt.wakeup t ()) w
+  if m.locked then begin
+    if Lwt_sequence.is_empty m.waiters then
+      m.locked <- false
+    else
+      Lwt.wakeup (Lwt_sequence.take_l m.waiters) ()
+  end
+
+let with_lock m f =
+  lock m >> begin
+    try_lwt
+      f ()
+    finally
+      unlock m;
+      return ()
+  end
