@@ -6,9 +6,9 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as
- * published by the Free Software Foundation, with linking exception;
+ * published by the Free Software Foundation, with linking exceptions;
  * either version 2.1 of the License, or (at your option) any later
- * version.
+ * version. See COPYING file for details.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,3 +26,10 @@ type t
 val create : unit -> t
 val lock : t -> unit Lwt.t
 val unlock : t -> unit
+
+val with_lock : t -> (unit -> 'a Lwt.t) -> 'a Lwt.t
+  (** [with_lock lock f] is used to lock a mutex within a block
+      scope.  The function [f ()] is called with the mutex locked, and
+      its result is returned from the call to {with_lock}. If an
+      exception is raised from f, the mutex is also unlocked before
+      the scope of {with_lock} is exited. *)
